
           LIST P=16F628           ;  16F628 Runs at 4.000 MHz
           INCLUDE "p16f628.inc"
  ; ==============
	__CONFIG _CP_OFF & _WDT_OFF & _PWRTE_ON & _HS_OSC & _LVP_OFF & _BODEN_OFF & _MCLRE_OFF        

    

;  Define Information
     #DEFINE RS PORTA, 0
     #DEFINE E  PORTA, 1
     
;  Macro

EStrobe MACRO                   ;  Strobe the "E" Bit
  bsf    E
  bcf    E
 ENDM

            CBLOCK     0x20
                TTt
                Time1
                Dlay            ; 8 Bit Delay Variable
                Temp            ; a temporary variable
                cntmsec
                LED1
                LED2
                LED3
                LED4
                LED5
                NumH
                NumL
                Tens
                Ones
                Thou
                TenK
                Hund
                Temp1
                Temp2
                Temp3
                Temp4
                SelFr
                FrLow
                FrHig
               ENDC
       
               ORG 4               ; start at location 0
 
            goto main           ; jump over to main routine       

            
ClearVar             
           clrf LED1
           clrf LED2
           clrf LED3
           clrf LED4
           clrf LED5
           clrf NumH
           clrf NumL
           clrf Tens
           clrf Ones
           clrf Thou
           clrf TenK
           clrf Hund 
           clrf SelFr
                    
           return

;----------------------------------------------------------------------;
;            Initialize the ports and timer1 an ADC                    ;
;----------------------------------------------------------------------;
init:
            
           clrf   PORTA
            movlw  0x07
            movwf  CMCON
            bsf    STATUS,RP0
            movlw  0x00                ;   
            movwf  TRISA
            movlw  0x40              ;    
            movwf  TRISB
            bcf    STATUS,RP0
            clrf   PORTA
            clrf   PORTB
            movlw  B'00000110'
            movwf  T1CON
            bcf    T1CON,0
          
            return



;----------------------------------------------------------------------;
;                 Initialize the LCD                                   ;
;----------------------------------------------------------------------;
initlcd:
  movlw D'40'
  call   nmsec                  ;  Wait 40 msecs before Reset
  bcf    RS                     ;  send an 8 bit instruction
  movlw  0x03                   ;  Reset Command
  call   NybbleOut              ;  Send the Nybble
  call   Dlay5                  ;  Wait 5 msecs before Sending Again
  EStrobe
  call Dlay160                  ;  Wait 160 usecs before Sending 2nd Time
  EStrobe
  call Dlay160                  ;  Wait 160 usecs before Sending 3rd Time
  bcf    RS                     ;  send an 8 bit instruction
  movlw  0x02                   ;  Set 4 Bit Mode
  call   NybbleOut              
  call Dlay160
  movlw  0x028                  ;  4 bit, 2 Line, 5x7 font
  call   SendINS
  movlw  0x010                  ;  display shift off
  call   SendINS
  movlw  0x001                  ;  Clear the Display RAM
  call   SendINS
  call   Dlay5                  ;  Note, Can take up to 4.1 msecs
 
  return

;----------------------------------------------------------------------;
;              Send the character in W out to the LCD                  ;
;----------------------------------------------------------------------;
SendASCII
                                ;  Send nbr as ASCII character
                                ;  Send the Character to the LCD
  movwf  Temp                   ;  Save the Temporary Value
  swapf  Temp, w                ;  Send the High Nybble
  bsf    RS                     ;  RS = 1
  call   NybbleOut
  movf   Temp, w                ;  Send the Low Nybble
  bsf    RS
  call   NybbleOut
  return

;----------------------------------------------------------------------;
;              Send an instruction in W out to the LCD                 ;
;----------------------------------------------------------------------;
SendINS                         ;  Send the Instruction to the LCD
  movwf  Temp                   ;  Save the Temporary Value
  swapf  Temp, w                ;  Send the High Nybble
  bcf    RS                     ;  RS = 0
  call   NybbleOut
  movf   Temp, w                ;  Send the Low Nybble
  bcf    RS
  call   NybbleOut
  return

;----------------------------------------------------------------------;
;              Send the nibble in W out to the LCD                     ;
;----------------------------------------------------------------------;
NybbleOut                       ;  Send a Nybble to the LCD
  andlw  B'00001111'
  movwf  PORTB                            
  EStrobe                       ;  Strobe out the LCD Data
  call Dlay160                  ;  delay for 160 usec
  return


;----------------------------------------------------------------------;
;                        time delay routines                           ;
;----------------------------------------------------------------------;

Dlay160:    movlw D'41'                ;  delay about 160 usec
micro4      addlw H'FF'                ;  subtract 1 from 'W'
            btfss STATUS,Z             ;  skip when you reach zero
            goto micro4                ;  more loops
            return                     

Dlay5:      movlw 5                    ;  delay for 5 milliseconds
            goto $ + 2
msec250:    movlw D'250'               ;  delay for 250 milliseconds
                ;*** N millisecond delay routine ***
nmsec:      movwf cntmsec              ; delay for N (in W) millisec
msecloop:   movlw D'254'               ; load takes .9765625 microsec
            call micro4                ; by itself CALL takes ...
                                       ; 2 + 253 X 4 + 3 + 2 = 1019 
                                       ; 1019 * .977 = 995 microsec
            nop                        ; .98 microsec 
            decfsz cntmsec, f          ; .98 skip not taken, else 1.95
            goto msecloop              ; 1.95 here: total ~1000 / loop
            return                     ; final time through ~999 to here
                                       ; overhead in and out ignored
;----------------------------------------------------------------------;
;                        time delay 200uS                              ;
;----------------------------------------------------------------------;
Delay200
         movlw D'100'
         movwf Temp2
         decfsz Temp2,1
         goto $-1
         return
;----------------------------------------------------------------------;
;                              LCD  Line1                              ;
;----------------------------------------------------------------------;
Line1
       movlw  0x080                 
       call   SendINS
       movlw  0x00C                  
       call   SendINS
       return
Line2
       movlw  " "
       call   SendASCII
       movlw  " "
       call   SendASCII
       movlw  " "
       call   SendASCII
       movlw  " "
       call   SendASCII
       movlw  " "
       call   SendASCII       
       movlw  " "
       call   SendASCII
       movlw  " "
       call   SendASCII
       movlw  " "
       call   SendASCII

       movlw  0x0C0                 
       call   SendINS
       movlw  0x00C                  
       call   SendINS
       return

;-----------------------------------------------------------------------;
;                Out to LCD frequence xxx.xx kHz                       ;
;-----------------------------------------------------------------------; 
FreqkHz
       
       movf  LED1,0
       addlw "0"
       call  SendASCII 
       movf  LED2,0
       addlw "0"
       call  SendASCII 
       movf  LED3,0
       addlw "0"
       call  SendASCII
       call  Line2 
       movlw "."
       call  SendASCII
       movf  LED4,0
       addlw "0"
       call  SendASCII 
       movf  LED5,0
       addlw "0"
       call  SendASCII 
       movlw " "
       call  SendASCII
 
       movlw "k"
       call  SendASCII
       movlw "H"
       call  SendASCII
       movlw "z"
       call  SendASCII
       return

;-----------------------------------------------------------------------;
;                Out to LCD frequence xxxxx Hz                       ;
;-----------------------------------------------------------------------; 
FreqHz
       
       movf  LED1,0
       addlw "0"
       call  SendASCII 
       movf  LED2,0
       addlw "0"
       call  SendASCII 
       movf  LED3,0
       addlw "0"
       call  SendASCII
       call  Line2
       movf  LED4,0
       addlw "0"
       call  SendASCII 
       movf  LED5,0
       addlw "0"
       call  SendASCII 
       movlw " "
       call  SendASCII

       movlw " "
       call  SendASCII
       movlw "H"
       call  SendASCII
       movlw "z"
       call  SendASCII
       movlw " "
       call  SendASCII
       return

;-----------------------------------------------------------------------;
;                Out to LCD frequence x.xxxx MHz                       ;
;-----------------------------------------------------------------------; 
FreqMHz1
       
       movf  LED1,0
       addlw "0"
       call  SendASCII
       movlw "."
       call  SendASCII 
       movf  LED2,0
       addlw "0"
       call  SendASCII
       call  Line2 
       movf  LED3,0
       addlw "0"
       call  SendASCII 
       movf  LED4,0
       addlw "0"
       call  SendASCII 
       movf  LED5,0
       addlw "0"
       call  SendASCII 
       movlw " "
       call  SendASCII

       movlw "M"
       call  SendASCII
       movlw "H"
       call  SendASCII
       movlw "z"
       call  SendASCII
       return

;-----------------------------------------------------------------------;
;                Out to LCD frequence xx.xxx MHz                       ;
;-----------------------------------------------------------------------; 
FreqMHz2
       
       movf  LED1,0
       addlw "0"
       call  SendASCII
       movf  LED2,0
       addlw "0"
       call  SendASCII
       movlw "."
       call  SendASCII
       call  Line2 
       movf  LED3,0
       addlw "0"
       call  SendASCII 
       movf  LED4,0
       addlw "0"
       call  SendASCII 
       movf  LED5,0
       addlw "0"
       call  SendASCII 
       movlw " "
       call  SendASCII

       movlw "M"
       call  SendASCII
       movlw "H"
       call  SendASCII
       movlw "z"
       call  SendASCII
       return


;----------------------------------------------------------------------;
;                            Text "Fin: "                              ;
;----------------------------------------------------------------------;       
Text1
       movlw  "F"
       call  SendASCII
       movlw  "r"
       call  SendASCII
       movlw  "q"
       call  SendASCII
       movlw  ":"
       call  SendASCII
       movlw  " "
       call  SendASCII
       return
;----------------------------------------------------------------------;
;                Convert two bute to dec                               ;
;----------------------------------------------------------------------;
Convert			
	movf    NumH,0		
	swapf	NumH,w
	andlw	0x0F 	
	addlw	0xF0 	
	movwf	Thou
	addwf	Thou,f
	addlw	0xE2
	movwf	Hund
	addlw	0x32
	movwf	Ones

	movf	NumH,w
	andlw	0x0F
	addwf	Hund,f
	addwf	Hund,f
	addwf	Ones,f
	addlw	0xE9
	movwf	Tens
	addwf	Tens,f
	addwf	Tens,f

	swapf	NumL,w
	andlw	0x0F
	addwf	Tens,f
	addwf	Ones,f

	rlf	Tens,f
	rlf	Ones,f
	comf	Ones,f
	rlf	Ones,f

	movf	NumL,w
	andlw	0x0F
	addwf	Ones,f
	rlf	Thou,f

	movlw	0x07
	movwf	TenK


	movlw	0A	 
Lb1:
	addwf	Ones,f
	decf	Tens,f
	btfss	3,0
	goto	Lb1
Lb2:
	addwf	Tens,f
	decf	Hund,f
	btfss	3,0
	goto	Lb2
Lb3:
	addwf	Hund,f
	decf	Thou,f
	btfss	3,0
	goto	Lb3
Lb4:
	addwf	Thou,f
	decf	TenK,f
	btfss	3,0
	goto	Lb4
	retlw   00

;----------------------------------------------------------------------;
;                Get frequence to LED1...LED5                          ;
;----------------------------------------------------------------------;
OutFrDis
       
        movf    FrHig,0
        movwf   NumH
        movf    FrLow,0
        movwf   NumL
      
        call    Convert
        movf    TenK,0
        movwf   LED1
        movf    Thou,0
        movwf   LED2
        movf    Hund,0
        movwf   LED3
        movf    Tens,0
        movwf   LED4
        movf    Ones,0
        movwf   LED5
        return


;----------------------------------------------------------------------;
;                Timer1 count pulse in RC0  Hz                         ;
;----------------------------------------------------------------------;
Count_Hz       
      
       clrf    FrLow
       clrf    FrHig
       movlw   .10
       movwf   TTt
       movlw   .176
       movwf   Temp1
       clrf    TMR1H
       clrf    TMR1L
       bcf     PIR1,0
       bsf     T1CON,0
DelayT100
       movlw   .189
       movwf   Temp2
       decfsz  Temp2,1
       goto    $-1
       decfsz  Temp1,1
       goto    DelayT100
       movlw   .175
       movwf   Temp1
       decfsz  TTt,1
       goto    DelayT100
       movlw   .50
       movwf   Temp3
       decfsz  Temp3,1
       goto    $-1
       bcf     T1CON,0
       movf    TMR1H,0
       movwf   FrHig
       movf    TMR1L,0
       movwf   FrLow
       clrf    SelFr
       return
;----------------------------------------------------------------------;
;                Timer1 count pulse in RC0  kHz                        ;
;----------------------------------------------------------------------;
Count_kHz       
       
       clrf    FrLow
       clrf    FrHig
       movlw   .174
       movwf   Temp1
       clrf    TMR1H
       clrf    TMR1L
       bcf     PIR1,0
       bsf     T1CON,0
DelayT10
       movlw   .190
       movwf   Temp2
       decfsz  Temp2,1
       goto    $-1
       decfsz  Temp1,1
       goto    DelayT10
       movlw   .41
       movwf   Temp3
       decfsz  Temp3,1
       goto    $-1
       bcf     T1CON,0
       movf    TMR1H,0
       movwf   FrHig
       movf    TMR1L,0
       movwf   FrLow
       movlw   .1
       movwf   SelFr
       return
;----------------------------------------------------------------------;
;                Timer1 count pulse in RC0  xMHz                       ;
;----------------------------------------------------------------------;
Count_MHz1       
       
       clrf    FrLow
       clrf    FrHig
       movlw   .86
       movwf   Temp1
       clrf    TMR1H
       clrf    TMR1L
       bcf     PIR1,0
       bsf     T1CON,0
DelayT1
       movlw   .36
       movwf   Temp2
       decfsz  Temp2,1
       goto    $-1
       decfsz  Temp1,1
       goto    DelayT1
       movlw   .122
       movwf   Temp3
       decfsz  Temp3,1
       goto    $-1
       nop
       bcf     T1CON,0
       movf    TMR1H,0
       movwf   FrHig
       movf    TMR1L,0
       movwf   FrLow
       movlw   .2
       movwf   SelFr
       return
;----------------------------------------------------------------------;
;                Timer1 count pulse in RC0  xMHz                       ;
;----------------------------------------------------------------------;
Count_MHz2       
       movlw   B'00010110'
       movwf   T1CON
       clrf    FrLow
       clrf    FrHig
       bcf     PIR1,0
       clrf    TMR1H
       clrf    TMR1L
       bsf     T1CON,0
       movlw   .250
       movwf   Temp3
       decfsz  Temp3,1
       goto    $-1
       movlw   .250
       movwf   Temp3
       decfsz  Temp3,1
       goto    $-1
       movlw   .164
       movwf   Temp3
       decfsz  Temp3,1
       goto    $-1
       nop
       nop
       nop
       nop
       bcf     T1CON,0
       movf    TMR1H,0
       movwf   FrHig
       movf    TMR1L,0
       movwf   FrLow
       movlw   B'00000110'
       movwf   T1CON
       movlw   .3
       movwf   SelFr
       return
;----------------------------------------------------------------------;
;                           Zummer sound                               ;
;----------------------------------------------------------------------;
Zummer 
       movlw .50
       movwf Temp1
zret   bcf   PORTB,5  
       bsf   PORTB,4
       call   Delay200
       bsf   PORTB,5
       bcf   PORTB,4
       call   Delay200
       decfsz Temp1,1
       goto   zret
       return



;----------------------------------------------------------------------;
;                           The Main routine                           ;
;----------------------------------------------------------------------;


main:
       call ClearVar
       call init               ; initialize ports, set up timer
       call initlcd            ; initialize the LCD
       call Zummer
Restart
 
                 
       call  Count_Hz
       btfsc PIR1,0
       call  Count_kHz
       btfsc PIR1,0
       call  Count_MHz1
       btfsc PIR1,0
       call  Count_MHz2
       call  OutFrDis
       call  Line1
       call  Text1
       
       movlw .0
       xorwf SelFr,0
       btfsc STATUS,Z
       call  FreqHz
       movlw .1
       xorwf SelFr,0
       btfsc STATUS,Z
       call  FreqkHz
       movlw .2
       xorwf SelFr,0
       btfsc STATUS,Z
       call  FreqMHz1
       movlw .3
       xorwf SelFr,0
       btfsc STATUS,Z
       call  FreqMHz2
    
       
      
        
       goto  Restart  
       end 


